import * as utils from '@src/utils/httpUtils'

describe('httpUtils', () => {
  let xhr, requests, server

  before(function () {
    xhr = sinon.useFakeXMLHttpRequest()
    requests = []
    xhr.onCreate = function (req) {
      requests.push(req)
    }
    server = sinon.fakeServer.create()
  })

  after(function () {
    xhr.restore()
    server.restore()
  })

  it('should be able to get with success callback', () => {
    const then = sinon.spy()
    const always = sinon.spy()
    utils.getRequest('/some/path')
      .then(then)
      .always(always)
    server.requests[0].respond(
      200,
      {'Content-Type': 'application/json'},
      JSON.stringify([{id: 1, text: 'Provide examples', done: true}])
    )
    sinon.assert.called(then)
    sinon.assert.called(always)
  })

  it('should be able to get with fail callback', () => {
    const then = sinon.spy()
    const err = sinon.spy()
    const always = sinon.spy()
    utils.getRequest('/some/path')
      .then(then)
      .catch(err)
      .always(always)
    server.requests[1].respond(
      500,
      {'Content-Type': 'application/json'},
      JSON.stringify([{id: 1, text: 'Provide examples', done: true}])
    )
    sinon.assert.called(err)
    sinon.assert.called(always)
  })
})
